# -*- cperl -*-

package GDT_POS_Vs_Deps_Check;
BEGIN { import TredMacro; }

use strict;
use Encode;
use utf8;

=item IsProblematic($node?)

Check if the given node has a probably erroneous tag.

=cut

sub IsProblematic {
    my $node=$_[0] || $this;

    return 0 unless $node;
    return 0 if ($node->{tag} eq "\-");
    return 0 if ($node->{tag} =~ /[A-Z][A-Z]/);

    if (SbWrongCase($node)) {
      return 1;
    } elsif  (NonSbNominativeCase($node)) {
      return 2;
    } elsif  (AtrHeadDifferentCase ($node)) {
      return 3;
    } elsif (PnPeAc_IObj ($node)) {
      return 4;
    } elsif (AtrHeadDifferentGender ($node)) {
      return 13;
    } elsif (PnReGeAndNumber($node)) {
      return 15;
    } elsif (HtanNumber($node)) {
      return 16;
    } elsif (WrongPOS_AtrSameCaseWithHeadBeforeHead($node)) {
      return 20;
    } elsif (At_Pn_RelativeToHead($node)) {
      return 21;
    } elsif (VbMnNf_DepIsExw($node)) {
      return 22;
    } elsif (Aj_Ad_Confusion($node)) {
      return 23;
    } elsif (Vb02_No_Confusion($node)) {
      return 24;
    } elsif (PnReSbObjAndAd($node)) {
      return 25;
    }
    # print STDERR "FoundParent " . $node->{form} . "\n";
    return 0;
}


sub SbWrongCase {
  # υπήρξε/Pred/VbMnIdPa03SgXxPeAvXx δραματική/Atr/AjBaFeSgNm αύξηση/Sb/NoCmFeSgNm
    my $node=$_[0] || $this;
    if ((myCase($node->{tag}) !~ /(Xx|Nm)/) && ($node->{afun} =~ /^Sb/)) {
      return 1;
    }
    return 0;
}

sub NonSbNominativeCase {
  # το/Atr/AtDfNeSgNm Βέλγιο/Adv_Co/NoPrNeSgNm
    my $node=$_[0] || $this;
    if ((myCase($node->{tag}) eq "Nm") && ($node->{afun} !~ /^(Sb|Pnom|ExD|Atr|Adv)/)) {
      return 1;
    }
    return 0;
}

sub AtrHeadDifferentCase {
  # διεθνείς/Atr/AjBaMaPlNm οργανισμοί/Sb/NoCmMaPlNm
  # εκατοντάδες/Atr/NmCtFePlNmNo χιλιάδες/Atr/NmCtFePlNmNo γυναίκες/Sb/NoCmFePlNm
  my $node=$_[0] || $this;
  if ($node->{afun} =~ /^Atr/) {
    my $parent = findMyRealParent($node);
    if ($parent) {

      if  ($node->{tag} =~ /^No/) {
	return 0;
      }


      if ($node->{ord} < $parent->{ord} ) {
	my $case = myCase($node->{tag});
	my $pCase = myCase($parent->{tag});
	return 0 if ($case eq "Xx");
	return 0 if ($pCase eq "Xx");
	if ($case ne $pCase) {
	  return 1;
	}
      }
    }
  }
  return 0;

}


sub PnPeAc_IObj {
  #μας/IObj/PnPeMa01PlAcWe έδωσε/Atr/VbMnIdPa03SgXxPeAvXx
  my $node=$_[0] || $this;
  if (($node->{afun} =~ /^IObj/) && ( $node->{tag} =~ /PnP.*Ac/ )) {
    my $parent = findMyRealParent($node);
    if ($parent) {
       if (hasClausalDirectObject($parent)) {
       	return 0;
       }
    }
    return 1;
  }
  return 0;
}



sub AtrHeadDifferentGender {
  # # -----------------------τέσσερις------------------
  # # κεντρικά/Atr/AjBaNePlAc σχέδια/Adv/NoCmNePlAc περνούσαν/Pred/VbMnIdPa03PlXxIpAvXx τέσσερις/Atr_Co/NmCdFePlAcAj έως/Coord/AsPpSp πέντε/Atr_Co/NmCdMaPlNmAj μήνες/Sb/NoCmMaPlAc 
  # #   tag=NmCdFePlAcAj: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./ep-02-02-28_033.fs
  # #   tag=NmCdMaPlNmAj: ./ep-02-02-28_033.fs
  my $node=$_[0] || $this;
  if ($node->{afun} =~ /^Atr/) {

    my $parent = findMyRealParent($node);
    if ($parent) {
      if ($node->{ord} < $parent->{ord} ) {
	my $gender = myGender($node->{tag});
	my $pGender = myGender($parent->{tag});
	return 0 if ($gender eq "Xx");
	return 0 if ($pGender eq "Xx");
	if ($gender ne $pGender) {
	  return 1;
	}
      }
    }
  }
  return 0;
}


sub PnReGeAndNumber {
# # -----------------------που------------------
# # η/Atr/AtDfFeSgNm Αμερική/Sb_Co/NoPrFeSgNm (/AuxG/OPUNCT που/Obj/PnReMa03PlNmXx την/Obj/PnPeFe03SgAcWe είχαν/AuxV/VbMnIdPa03PlXxIpAvXx ονομάσει/Atr_Pa/VbMnNfXxXxXxXxPeAvXx 
# #   tag=PnReFe03SgAcXx: ./EU-History_of_Europe_1500_1900-2008XXXX.fs
# #   tag=PnReMa03PlNmXx: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./EU-History_of_Europe_1500_1900-2008XXXX.fs
  my $node=$_[0] || $this;
  if ($node->{tag} =~ /^PnRe/ && ($node->{lemma} eq "που")) {
    my $head = findMyClausesHead($node);
    if ($head) {
      my $gender = myGender($node->{tag});
      my $hGender = myGender($head->{tag});
      return 0 if ($gender eq "Xx");
      return 0 if ($hGender eq "Xx");
      if ($gender ne $hGender) {
	return 1;
      }

      my $number = myNumber($node->{tag});
      my $hNumber = myNumber($head->{tag});
      return 0 if ($number eq "Xx");
      return 0 if ($hNumber eq "Xx");
      if ($number ne $hNumber) {
	return 1;
      }
    }
  }
  return 0;
}


sub PnReSbObjAndAd {
  my $node=$_[0] || $this;
  if (($node->{lemma} eq "που") && ($node->{tag} =~ /^AdXx/) &&  ($node->{afun} =~ /(Sb|Ob)/)) {
      return 1;
  }
  return 0;
}






sub WrongPOS_AtrSameCaseWithHeadBeforeHead {
  # ιατρικής/Atr/NoCmFeSgGe φροντίδας/Atr_Co/NoCmFeSgGe

  my $node=$_[0] || $this;
  if ($node->{afun} =~ /^Atr/) {
    my $parent = findMyRealParent($node);
    if ($parent) {

      # Ισπανοί έμποροι || Απόστολο Παύλο
      if ( (($node->{tag} =~ /NoPr/) && ($parent->{tag} =~ /No/) )   ||
	(($parent->{tag} =~ /NoPr/) && ($node->{tag} =~ /No/) ) ) {
	return 0;
      }

      if (($node->{ord} < $parent->{ord} ) && ($node->{tag} =~ /^No/)) {
	my $case = myCase($node->{tag});
	my $pCase = myCase($parent->{tag});
	if ( $case eq $pCase ) {
	  return 1;
	}
      }
    }
  }
  return 0;
}

# sub AtrHeadDifferentNumber {
#   # Τσεχοσλοβακία------------------
#   # tag=NoPrFeSgAc: ./WikipediaEL-I_Germania_meta_to_1806-2009XXXX.fs
#   # tag=NoPrFePlAc:
# }

sub At_Pn_RelativeToHead {
  my $node=$_[0] || $this;
  if ($node->{tag} =~ /^At/ ) {
    my $parent = findMyRealParent($node);
    if ($parent) {
      if ($node->{ord} > $parent->{ord} ) {
	return 1;
      }
    }
  } elsif ($node->{tag} =~ /^PnPo/ ) {
    my $parent = findMyRealParent($node);
    if ($parent) {
      if ($node->{ord} < $parent->{ord} ) {
	return 1;
      }
    }
  } elsif ($node->{tag} =~ /^PnPe/ ) {
    my $parent = findMyRealParent($node);
    if ($parent) {
      if (($node->{ord} < $parent->{ord} )  && ($parent->{tag} !~ /^Vb/) ) {
	return 1;
      }
    }
  }
  return 0;
}

sub VbMnNf_DepIsExw {
  # # -----------------------τεθεί------------------
  # # δεν/AuxV/PtNg είχε/AuxV/VbMnIdPa03SgXxIpAvXx ποτέ/Adv/AdXxBa τεθεί/Pred/VbMnIdXx03SgXxPePvXx στην/AuxP/AsPpPaFeSgAc κρίση/IObj/NoCmFeSgAc του/Atr/AtDfMaSgGe 
  # #   tag=VbMnIdXx03SgXxPePvXx: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./WikipediaEL-Emfilios_Polemos-20090304.fs
  # #   tag=VbMnNfXxXxXxXxPeAvXx: ./WikipediaEL-Emfilios_Polemos-20090304.fs
  my $node=$_[0] || $this;
  my $result = 0;
  if ($node->{tag} =~ /^VbMnId.*03/) {
    foreach my $child ($node->children()) {
      if (($child->{lemma} eq "έχω" )  && ($child->{afun} =~ /AuxV/) ) {
	$result = 1;
      }
    }
  }
  return $result;
}

sub Aj_Ad_Confusion {
# #   -----------------------πολιτικά------------------
# # 1946/Adv/DATE οι/Atr/AtDfMaPlNm επιζώντες/Atr/NoCmMaPlNm πολιτικά/Atr_Co/AdXxBa και/Coord/CjCo στρατιωτικά/Atr_Co/AdXxBa κυρίως/Adv/AdXxBa 
# #   tag=AdXxBa: ./WikipediaEL-I_Germania_meta_to_1806-2009XXXX.fs
# #   tag=AjBaNePlAc: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./WikipediaEL-I_Germania_meta_to_1806-2009XXXX.fs
# # == 16 =============
# # ./WikipediaEL-I_Germania_meta_to_1806-2009XXXX.fs[16]: στρατιωτικός.
# # -----------------------στρατιωτικά------------------
  my $node=$_[0] || $this;

  if ($node->{tag} =~ /^Aj/ &&  $node->{afun} =~ /^Adv/ ) {
    return 1;
  } elsif ($node->{tag} =~ /^Ad/ &&  $node->{afun} =~ /^(Atr|Pnom)/ && (!$node->firstson())) {
    return 1;
  }

  return 0;

}

sub Vb02_No_Confusion {
# # -----------------------ανακαλύψεις------------------
# # Οι/Atr/AtDfFePlNm μεγάλες/Atr/AjBaFePlNm ανακαλύψεις/ExD_Co/VbMnIdXx02SgXxPeAvXx και/Coord_Co/CjCo οι/Atr/AtDfFePlNm νέες/Atr/AjBaFePlNm 
# #   tag=NoCmFePlNm: ./EU-History_of_Europe_1500_1900-2008XXXX.fs
# #   tag=VbMnIdXx02SgXxPeAvXx: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./EU-History_of_Europe_1500_1900-2008XXXX.fs
  my $node=$_[0] || $this;
  if ($node->{tag} =~ /^Vb.*02Sg/) {
    foreach my $child ($node->children()) {
      if ($child->{tag} =~ /^(At|Aj)/ ) {
	return 1;
      }
    }
  }
  return 0;
}

# sub Aj2VbIfPred {
# # -----------------------διαρκούσαν------------------
# # ήταν/Pred_Co/VbMnIdPa03SgXxIpPvXx δύσκολα/Pnom/AdXxBa και/Coord_Co/CjCo διαρκούσαν/Pred_Co/AjBaFeSgAc πολύ/Adv/AdXxBa ,/AuxX/PUNCT γι'/AuxP/AsPpSp 
# #   tag=VbMnIdPa03PlXxPeAvXx: ./EU-History_of_Europe_1500_1900-2008XXXX.fs
# #   tag=AjBaFeSgAc: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./EU-History_of_Europe_1500_1900-2008XXXX.fs
# }

# sub NotAVerbButHasSubj {
#   # νικά/Adv/VbMnIdPr03SgXxIpAvXx
# }


sub HtanNumber {
# # -----------------------ήταν------------------
# # τα/Atr/AtDfNePlNm χερσαία/Atr/AjBaNePlNm ταξίδια/Sb/NoCmNePlNm ήταν/Pred_Co/VbMnIdPa03SgXxIpPvXx δύσκολα/Pnom/AdXxBa και/Coord_Co/CjCo διαρκούσαν/Pred_Co/AjBaFeSgAc 
# #   tag=VbMnIdPa03SgXxIpPvXx: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./EU-History_of_Europe_1500_1900-2008XXXX.fs
# #   tag=VbMnIdPa03PlXxIpPvXx: ./EU-History_of_Europe_1500_1900-2008XXXX.fs
  my $node=$_[0] || $this;
  if ($node->{lemma} eq "είμαι") {

    foreach my $child ($node->children()) {

      if (($child->{afun} =~ /Sb/) && ($node->{tag} !~ /Vb/) && (myNumber($node->{tag}) ne myNumber($child->{tag})) ){
	return 1;
      }
    }
  }

  return 0;
}

# sub No2Pred {

# # ./WikipediaEL-Kritiko_Zitima-20090408.fs[8]: αναστέλλω.
# # -----------------------ανεστάλη------------------
# # των/Atr/AtDfMaPlGe Κρητικών/Atr/NoPrMaPlGe προσωρινά/Adv/AdXxBa ανεστάλη/Pred/VbMnIdPa03SgXxPePvXx ./AuxK/PTERM_P 
# #   tag=NoCmFeSgAc: /usr/local/gdt/data/2009_ekpa/poscorrection/interanno_checks/toAnnotators//./WikipediaEL-Kritiko_Zitima-20090408.fs
# #   tag=VbMnIdPa03SgXxPePvXx: ./WikipediaEL-Kritiko_Zitima-20090408.fs

# }


sub myCase {
  my $tag = $_[0];
  # print STDERR "$tag\n";
  if ($tag =~ /^(No|At|Aj|Nm)/) { # NoPrFeSgNm
    return substr($tag, 8, 2);
  } elsif ($tag =~ /^Pn/) {# PnRe03FeSgNmXx
    return substr($tag, 10, 2);
  } elsif (($tag =~ /^VbMnPp/) && ($tag !~ /Xx$/)) { # VbMnPpXxXxSgMaPePvNm
    return substr($tag, 18, 2);
  } else {
    return "Xx";
  }
}


sub myGender {
  my $tag = $_[0];
  if ($tag =~ /^(No|At|Aj|Nm)/) { # NoPrFeSgNm
    return substr($tag, 4, 2);
  } elsif ($tag =~ /^Pn/) {# PnReFe03SgNmXx
    return substr($tag, 4, 2);
  } elsif (($tag =~ /^VbMnPp/) && ($tag !~ /.{12}Xx$/)) { # VbMnPpXxXxSgMaPePvNm
    return substr($tag, 12, 2);
  } else {
    return "Xx";
  }
}



sub myNumber {
  my $tag = $_[0];
  if ($tag =~ /^(No|At|Aj|Nm)/) { # NoPrFeSgNm
    return substr($tag, 6, 2);
  } elsif ($tag =~ /^Pn/) {# PnReFe03SgNmXx
    return substr($tag, 8, 2);
  } elsif ($tag =~ /^Vb/) { # VbMnPpXxXxSgMaPePvNm
    return substr($tag, 10, 2);
  } else {
    return "Xx";
  }
}


sub findMyRealParent {
    my $node=$_[0] || $this;
    return 0 unless $node;

    my $parent=$node->parent if ($node->parent);
    if ($node->{tag} =~ /^At/) {
      # Fixme Coord/Apos
      while ($parent->{afun} =~ /^(Coord|Apos|Aux)/) {
	if ($parent->firstson) {
	  $parent = $parent->firstson();
	} else {
	  last;
	}
      }
    } else {
      # Fixme Coord/Apos
      while ($parent->{afun} =~ /^(Coord|Apos|Aux)/) {
	if ($parent->parent) {
	  $parent = $parent->parent();
	} else {
	  last;
	}
      }
    }
    return $parent;
}

sub findMyClausesHead {
    my $node=$_[0] || $this;
    return 0 unless $node;
    my $parent=$node->parent if ($node->parent);
    # print STDERR "MyClause Head1: " . $node->{form} . " " . $parent->{form} . "\n";
    # Fixme Coord/Apos
    while ($parent->{tag} =~ /^(Vb|Coord)/  ) {
      if ($parent->parent) {
	$parent = $parent->parent();
      } else {
	last;
      }
    }

    # print STDERR "MyClause Head3: " . $node->{form} . " " . $parent->{form} . "\n";
    return $parent;
}


sub hasClausalDirectObject {
    my $node=$_[0] || $this;
    return 0 unless $node;

    my $result = 0;

    foreach my $child ($node->children()) {
      if (($child->{tag} =~ /^Vb/) && ($child->{afun} =~ /^Obj/)) {
	$result = 1;
	last;
      } else {
	$result = hasClausalDirectObject($child);
	if ($result == 1) {
	  last;
	}
      }
    }
    return $result;
}


1;
